@@ -1,4 +1,4 @@
 from isopy.toolbox.isotope import *
 from isopy.toolbox.regress import *
-from isopy.toolbox.plotting import *
+from isopy.toolbox.plot import *
 from isopy.toolbox.doublespike import *
\ No newline at end of file
